/**
 * Calcula quantidade de horas a faturar a partir de um 
 * CHT (controle de horas trabalhadas)
 * 
 * @author Vanrochris Helbert Vieira (08/09/2008) 
 *
 * @param P_CDCLIENTE Cdigo do cliente
 *        P_CDOPERAPUR Cdigo do operador
 *        P_NAC Identificador de clientes nacionais e/ou internacionais
 *        P_MES Ms de referncia da apurao
 
 *        P_NRPROPOSFAS Nmero da proposta
 *        P_NRVERPROPFAS Nmero da verso da proposta
 *        P_CDTIPOSERV Cdigo do tipo de servio
 *        P_NRSERVCONT Cdigo do servio
 *        P_NRSEQUCHT Nmero sequencial
 *
 * @return Quantidade de horas a faturar
 */

CREATE OR REPLACE FUNCTION CALCULA_FAT_MDO
(
   P_NRPROPOSFAS  IN  VARCHAR2,
   P_NRVERPROPFAS IN  VARCHAR2,
   P_CDTIPOSERV   IN  VARCHAR2,
   P_NRSERVCONT   IN  VARCHAR2,
   P_NRSEQUCHT    IN  VARCHAR2
)
RETURN NUMBER
IS
   V_VRFATU          NUMBER;
   V_QTHRAPROFATUCHT NUMBER;
   V_QTHREXTRSERVCHT NUMBER;
   V_QTSERVCONT      NUMBER;
   V_VRDEHORACONT    NUMBER;
   V_VRHORACLIE      NUMBER;
   V_VRPACOTESERV    NUMBER;
   V_IDTIPOCOBRCONT  VARCHAR2(1);
BEGIN
   BEGIN
      SELECT QTHRAPROFATUCHT, QTHREXTRSERVCHT
        INTO V_QTHRAPROFATUCHT, V_QTHREXTRSERVCHT
        FROM CHT
       WHERE NRPROPOSFAS = P_NRPROPOSFAS
         AND NRVERPROPFAS = P_NRVERPROPFAS
         AND CDTIPOSERV = P_CDTIPOSERV
         AND NRSERVCONT = P_NRSERVCONT
         AND NRSEQUCHT = P_NRSEQUCHT;
   EXCEPTION
      WHEN NO_DATA_FOUND THEN
         NULL;
   END;
   BEGIN
      SELECT C.IDTIPOCOBRCONT, NVL(C.QTSERVCONT, 0), NVL(ROUND(C.VRDEHORACONT,2), NVL(ROUND(V.VRHORACLIE,2), 0)), NVL(ROUND(V.VRHORACLIE,2), 0), NVL(ROUND(P.VRPACOTESERV,2),0)
        INTO V_IDTIPOCOBRCONT, V_QTSERVCONT, V_VRDEHORACONT, V_VRHORACLIE, V_VRPACOTESERV
        FROM CONTRATOSERV C, VALORSERV V, PROPOSTAFAS P,
            (SELECT NRPROPOSFAS, NRVERPROPFAS, NRSERVCONT, CDTIPOSERV,
                    MAX(DTFVIGVRSERV) AS DTFVIGVRSERV
               FROM VALORSERV
              GROUP BY NRPROPOSFAS, NRVERPROPFAS, NRSERVCONT, CDTIPOSERV) M
       WHERE C.NRPROPOSFAS = P_NRPROPOSFAS
         AND C.NRVERPROPFAS = P_NRVERPROPFAS
         AND C.CDTIPOSERV = P_CDTIPOSERV
         AND C.NRSERVCONT = P_NRSERVCONT
         AND C.NRPROPOSFAS = V.NRPROPOSFAS
         AND C.NRVERPROPFAS = V.NRVERPROPFAS
         AND C.CDTIPOSERV = V.CDTIPOSERV
         AND C.NRSERVCONT = V.NRSERVCONT
         AND V.NRPROPOSFAS = M.NRPROPOSFAS
         AND V.NRVERPROPFAS = M.NRVERPROPFAS
         AND V.NRSERVCONT = M.NRSERVCONT
         AND V.CDTIPOSERV = M.CDTIPOSERV
         AND V.DTFVIGVRSERV = M.DTFVIGVRSERV
         AND C.NRPROPOSFAS = P.NRPROPOSFAS
         AND C.NRVERPROPFAS = P.NRVERPROPFAS;
   EXCEPTION
      WHEN NO_DATA_FOUND THEN
         NULL;
   END;

   IF V_IDTIPOCOBRCONT = 'F' THEN -- Horas fixas 
      V_VRFATU := (V_QTSERVCONT * V_VRDEHORACONT);
   ELSE IF V_IDTIPOCOBRCONT = 'P' THEN -- Pacote
         V_VRFATU := (V_VRPACOTESERV);
      ELSE -- Proporcional s horas dos CDAs correspondentes ao CHT
         V_VRFATU := (V_QTHRAPROFATUCHT * V_VRDEHORACONT) + (NVL(V_QTHREXTRSERVCHT, 0) * 1.5 * V_VRDEHORACONT);
      END IF;
   END IF;
   RETURN(V_VRFATU);
END;
/

-- VERSAO --
BEGIN ATUVEROBJ('CALCULA_FAT_MDO','5.05.004'); END;
/
